@@ -5,7 +5,7 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
 		       n.omp.threads = 1, verbose = TRUE,  
 		       n.report = 100, 
 		       n.burn = round(.10 * n.batch * batch.length),
-		       n.thin = 1, k.fold, k.fold.threads = 1, 
+		       n.thin = 1, n.chains = 1, k.fold, k.fold.threads = 1, 
 		       k.fold.seed = 100, k.fold.data, ...){
 
   ptm <- proc.time()
@@ -584,9 +584,9 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
 
     # Set model.deviance to NA for returning when no cross-validation
     model.deviance <- NA
+    curr.chain <- 1
 
   if (!NNGP) { 
-
     # Specify storage modes -----------------------------------------------
     storage.mode(y) <- "double"
     storage.mode(z.inits) <- "double"
@@ -631,52 +631,91 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
     storage.mode(n.report) <- "integer"
     storage.mode(n.burn) <- "integer"
     storage.mode(n.thin) <- "integer"
+    storage.mode(curr.chain) <- "integer"
+    storage.mode(n.chains) <- "integer"
     n.post.samples <- length(seq(from = n.burn + 1, 
 				 to = n.samples, 
 				 by = as.integer(n.thin)))
     storage.mode(n.post.samples) <- "integer"
 
-    # Run the model in C    
-    out <- .Call("spIntPGOcc", y, X, X.p.all, coords.D, p.occ, p.det, p.det.long, 
-		 J, J.long, K, n.obs, n.obs.long, n.data, 
-		 beta.inits, alpha.inits, z.inits, w.inits, 
-		 phi.inits, sigma.sq.inits, nu.inits, 
-		 z.long.indx.c, data.indx.c, alpha.indx.c, mu.beta, mu.alpha, 
-		 Sigma.beta, sigma.alpha, phi.a, phi.b, sigma.sq.a, sigma.sq.b, 
-		 nu.a, nu.b, tuning.c, cov.model.indx, 
-		 n.batch, batch.length, accept.rate,  
-		 n.omp.threads, verbose, n.report, n.burn, n.thin, n.post.samples)
+    out.tmp <- list()
+    for (i in 1:n.chains) {
+      # Change initial values if i > 1
+      if (i > 1) {
+        beta.inits <- rnorm(p.occ, mu.beta, sqrt(sigma.beta))
+        alpha.inits <- rnorm(p.det, mu.alpha, sqrt(sigma.alpha))
+        sigma.sq.inits <- rigamma(1, sigma.sq.a, sigma.sq.b)
+        phi.inits <- runif(1, phi.a, phi.b)
+	if (cov.model == 'matern') {
+          nu.inits <- runif(1, nu.a, nu.b)
+	}
+      }
+      storage.mode(curr.chain) <- "integer" 
+      # Run the model in C
+      out.tmp[[i]] <- .Call("spIntPGOcc", y, X, X.p.all, coords.D, p.occ, p.det, p.det.long, 
+		            J, J.long, K, n.obs, n.obs.long, n.data, 
+		            beta.inits, alpha.inits, z.inits, w.inits, 
+		            phi.inits, sigma.sq.inits, nu.inits, 
+		            z.long.indx.c, data.indx.c, alpha.indx.c, mu.beta, mu.alpha, 
+		            Sigma.beta, sigma.alpha, phi.a, phi.b, sigma.sq.a, sigma.sq.b, 
+		            nu.a, nu.b, tuning.c, cov.model.indx, 
+		            n.batch, batch.length, accept.rate,  
+		            n.omp.threads, verbose, n.report, n.burn, n.thin, n.post.samples, 
+			    curr.chain, n.chains)
+      curr.chain <- curr.chain + 1
+    }
+    # Calculate R-Hat ---------------
+    out <- list()
+    out$rhat <- list()
+    if (n.chains > 1) {
+      out$rhat$beta <- gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+      					      mcmc(t(a$beta.samples)))), 
+      			     autoburnin = FALSE)$psrf[, 2]
+      out$rhat$alpha <- gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+      					      mcmc(t(a$alpha.samples)))), 
+      			      autoburnin = FALSE)$psrf[, 2]
+      out$rhat$theta <- gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+						      mcmc(t(a$theta.samples)))), 
+			            autoburnin = FALSE)$psrf[, 2]
 
-    out$beta.samples <- mcmc(t(out$beta.samples))
+    } else {
+      out$rhat$beta <- rep(NA, p.occ)
+      out$rhat$alpha <- rep(NA, p.det)
+      out$rhat$theta <- rep(NA, ifelse(cov.model == 'matern', 3, 2))
+    }
+
+    out$beta.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$beta.samples))))
     colnames(out$beta.samples) <- x.names
-    out$alpha.samples <- mcmc(t(out$alpha.samples))
+    out$alpha.samples <- mcmc(do.call(rbind, 
+      				lapply(out.tmp, function(a) t(a$alpha.samples))))
     colnames(out$alpha.samples) <- x.p.names
-    out$theta.samples <- mcmc(t(out$theta.samples))
+    out$theta.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$theta.samples))))
     if (cov.model != 'matern') {
       colnames(out$theta.samples) <- c('sigma.sq', 'phi')
     } else {
       colnames(out$theta.samples) <- c('sigma.sq', 'phi', 'nu')
     }
-    if (cov.model != 'matern') {
-      colnames(out$theta.samples) <- c('sigma.sq', 'phi')
-    } else {
-      colnames(out$theta.samples) <- c('sigma.sq', 'phi', 'nu')
-    }
-    out$z.samples <- mcmc(t(out$z.samples))
-    out$w.samples <- mcmc(t(out$w.samples))
-    out$psi.samples <- mcmc(t(out$psi.samples))
+    out$z.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$z.samples))))
+    out$psi.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$psi.samples))))
+    out$w.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$w.samples))))
     # y.rep.samples is returned as a list, where each element 
+    out$y.rep.samples <- do.call(rbind, lapply(out.tmp, function(a) a$y.rep.samples))
     # corresponds to a different data set. 
     tmp <- list()
     indx <- 1
     for (q in 1:n.data) {
-      tmp[[q]] <- array(NA, dim = c(J.long[q] * K.long.max[q], n.post.samples))
+      tmp[[q]] <- array(NA, dim = c(J.long[q] * K.long.max[q], n.post.samples * n.chains))
       tmp[[q]][names.long[[q]], ] <- out$y.rep.samples[indx:(indx + n.obs.long[q] - 1), ] 
-      tmp[[q]] <- array(tmp[[q]], dim = c(J.long[q], K.long.max[q], n.post.samples))
+      tmp[[q]] <- array(tmp[[q]], dim = c(J.long[q], K.long.max[q], n.post.samples * n.chains))
       tmp[[q]] <- aperm(tmp[[q]], c(3, 1, 2))
       indx <- indx + n.obs.long[q]
     }
     out$y.rep.samples <- tmp
+    # Calculate effective sample sizes
+    out$ESS <- list()
+    out$ESS$beta <- effectiveSize(out$beta.samples)
+    out$ESS$alpha <- effectiveSize(out$alpha.samples)
+    out$ESS$theta <- effectiveSize(out$theta.samples)
     out$X <- X
     out$X.p <- X.p.orig
     out$y <- y.big
@@ -689,6 +728,7 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
     out$n.post <- n.post.samples
     out$n.thin <- n.thin
     out$n.burn <- n.burn
+    out$n.chains <- n.chains
 
     # K-fold cross-validation -------
     if (!missing(k.fold)) {
@@ -846,7 +886,8 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
         storage.mode(n.report) <- "integer"
         storage.mode(n.burn) <- "integer"
         storage.mode(n.thin) <- "integer"
-
+	curr.chain <- 1
+	storage.mode(curr.chain) <- "integer"
 
         out.fit <- .Call("spIntPGOcc", y.fit, X.fit, X.p.fit, coords.D.fit, p.occ, p.det, p.det.long, 
 		         J.fit, J.long.fit, K.fit, n.obs.fit, n.obs.long.fit, n.data, 
@@ -856,7 +897,8 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
 		         Sigma.beta, sigma.alpha, phi.a, phi.b, sigma.sq.a, sigma.sq.b, 
 		         nu.a, nu.b, tuning.c, cov.model.indx, 
 		         n.batch, batch.length, accept.rate,  
-		         n.omp.threads.fit, verbose.fit, n.report, n.burn, n.thin, n.post.samples)
+		         n.omp.threads.fit, verbose.fit, n.report, n.burn, n.thin, n.post.samples, 
+			 curr.chain, n.chains)
 
         out.fit$beta.samples <- mcmc(t(out.fit$beta.samples))
         colnames(out.fit$beta.samples) <- x.names
@@ -867,6 +909,8 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
           colnames(out.fit$theta.samples) <- c('sigma.sq', 'phi', 'nu')
         }
         out.fit$w.samples <- mcmc(t(out.fit$w.samples))
+	out.fit$z.samples <- mcmc(t(out.fit$z.samples))
+	out.fit$psi.samples <- mcmc(t(out.fit$psi.samples))
         out.fit$X <- X.fit
         out.fit$X.p <- X.p.fit
         out.fit$call <- cl
@@ -877,6 +921,7 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
         out.fit$n.post <- n.post.samples
         out.fit$n.thin <- n.thin
         out.fit$n.burn <- n.burn
+	out.fit$n.chains <- 1
 	class(out.fit) <- "spPGOcc"
 
         out.pred <- predict.spPGOcc(out.fit, X.0, coords.0, verbose = FALSE)
@@ -991,51 +1036,99 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
     storage.mode(n.neighbors) <- "integer"
     storage.mode(n.burn) <- "integer"
     storage.mode(n.thin) <- "integer"
+    storage.mode(curr.chain) <- "integer"
+    storage.mode(n.chains) <- "integer"
     n.post.samples <- length(seq(from = n.burn + 1, 
 				 to = n.samples, 
 				 by = as.integer(n.thin)))
     storage.mode(n.post.samples) <- "integer"
 
-    # Run the model in C    
-    out <- .Call("spIntPGOccNNGP", y, X, X.p.all, coords, p.occ, p.det, p.det.long, 
-		 J, J.long, K, n.obs, n.obs.long, n.data, 
-		 n.neighbors, nn.indx, nn.indx.lu, u.indx, u.indx.lu, ui.indx, 
-		 beta.inits, alpha.inits, z.inits, w.inits, 
-		 phi.inits, sigma.sq.inits, nu.inits, 
-		 z.long.indx.c, data.indx.c, alpha.indx.c, mu.beta, mu.alpha, 
-		 Sigma.beta, sigma.alpha, phi.a, phi.b, sigma.sq.a, sigma.sq.b, 
-		 nu.a, nu.b, tuning.c, cov.model.indx,
-		 n.batch, batch.length, accept.rate,  
-		 n.omp.threads, verbose, n.report, n.burn, n.thin, n.post.samples)
+    out.tmp <- list()
+    for (i in 1:n.chains) {
+      # Change initial values if i > 1
+      if (i > 1) {
+        beta.inits <- rnorm(p.occ, mu.beta, sqrt(sigma.beta))
+        alpha.inits <- rnorm(p.det, mu.alpha, sqrt(sigma.alpha))
+        sigma.sq.inits <- rigamma(1, sigma.sq.a, sigma.sq.b)
+        phi.inits <- runif(1, phi.a, phi.b)
+	if (cov.model == 'matern') {
+          nu.inits <- runif(1, nu.a, nu.b)
+	}
+      }
+      storage.mode(curr.chain) <- "integer" 
+      # Run the model in C
+      out.tmp[[i]] <- .Call("spIntPGOccNNGP", y, X, X.p.all, coords, p.occ, p.det, p.det.long, 
+		            J, J.long, K, n.obs, n.obs.long, n.data, 
+		            n.neighbors, nn.indx, nn.indx.lu, u.indx, u.indx.lu, ui.indx, 
+		            beta.inits, alpha.inits, z.inits, w.inits, 
+		            phi.inits, sigma.sq.inits, nu.inits, 
+		            z.long.indx.c, data.indx.c, alpha.indx.c, mu.beta, mu.alpha, 
+		            Sigma.beta, sigma.alpha, phi.a, phi.b, sigma.sq.a, sigma.sq.b, 
+		            nu.a, nu.b, tuning.c, cov.model.indx,
+		            n.batch, batch.length, accept.rate,  
+		            n.omp.threads, verbose, n.report, n.burn, n.thin, n.post.samples, 
+			    curr.chain, n.chains)
+      curr.chain <- curr.chain + 1
+    }
+    # Calculate R-Hat ---------------
+    out <- list()
+    out$rhat <- list()
+    if (n.chains > 1) {
+      out$rhat$beta <- gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+      					      mcmc(t(a$beta.samples)))), 
+      			     autoburnin = FALSE)$psrf[, 2]
+      out$rhat$alpha <- gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+      					      mcmc(t(a$alpha.samples)))), 
+      			      autoburnin = FALSE)$psrf[, 2]
+      out$rhat$theta <- gelman.diag(mcmc.list(lapply(out.tmp, function(a) 
+						      mcmc(t(a$theta.samples)))), 
+			            autoburnin = FALSE)$psrf[, 2]
+
+    } else {
+      out$rhat$beta <- rep(NA, p.occ)
+      out$rhat$alpha <- rep(NA, p.det)
+      out$rhat$theta <- rep(NA, ifelse(cov.model == 'matern', 3, 2))
+    }
 
     # Get everything back in the original order
     out$coords <- coords[order(ord), ]
-    out$z.samples <- mcmc(t(out$z.samples[order(ord), , drop = FALSE]))
+    out$z.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$z.samples))))
+    out$z.samples <- mcmc(out$z.samples[, order(ord), drop = FALSE])
     out$X <- X[order(ord), , drop = FALSE]
-    out$w.samples <- mcmc(t(out$w.samples[order(ord), , drop = FALSE]))
-    out$psi.samples <- mcmc(t(out$psi.samples[order(ord), , drop = FALSE]))
-
-    out$beta.samples <- mcmc(t(out$beta.samples))
+    out$w.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$w.samples))))
+    out$w.samples <- mcmc(out$w.samples[, order(ord), drop = FALSE])
+    out$psi.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$psi.samples))))
+    out$psi.samples <- mcmc(out$psi.samples[, order(ord), drop = FALSE])
+    out$beta.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$beta.samples))))
     colnames(out$beta.samples) <- x.names
-    out$alpha.samples <- mcmc(t(out$alpha.samples))
+    out$alpha.samples <- mcmc(do.call(rbind, 
+      				lapply(out.tmp, function(a) t(a$alpha.samples))))
     colnames(out$alpha.samples) <- x.p.names
-    out$theta.samples <- mcmc(t(out$theta.samples))
+    out$theta.samples <- mcmc(do.call(rbind, lapply(out.tmp, function(a) t(a$theta.samples))))
     if (cov.model != 'matern') {
       colnames(out$theta.samples) <- c('sigma.sq', 'phi')
     } else {
       colnames(out$theta.samples) <- c('sigma.sq', 'phi', 'nu')
     }
-    # Get y in a useful format. 
+    # y.rep.samples is returned as a list, where each element 
+    out$y.rep.samples <- do.call(rbind, lapply(out.tmp, function(a) a$y.rep.samples))
+    # corresponds to a different data set. 
     tmp <- list()
     indx <- 1
     for (q in 1:n.data) {
-      tmp[[q]] <- array(NA, dim = c(J.long[q] * K.long.max[q], n.post.samples))
+      tmp[[q]] <- array(NA, dim = c(J.long[q] * K.long.max[q], n.post.samples * n.chains))
       tmp[[q]][names.long[[q]], ] <- out$y.rep.samples[indx:(indx + n.obs.long[q] - 1), ] 
-      tmp[[q]] <- array(tmp[[q]], dim = c(J.long[q], K.long.max[q], n.post.samples))
+      tmp[[q]] <- array(tmp[[q]], dim = c(J.long[q], K.long.max[q], n.post.samples * n.chains))
       tmp[[q]] <- aperm(tmp[[q]], c(3, 1, 2))
       indx <- indx + n.obs.long[q]
     }
     out$y.rep.samples <- tmp
+    # Calculate effective sample sizes
+    out$ESS <- list()
+    out$ESS$beta <- effectiveSize(out$beta.samples)
+    out$ESS$alpha <- effectiveSize(out$alpha.samples)
+    out$ESS$theta <- effectiveSize(out$theta.samples)
+
     out$X.p <- X.p.orig
     out$y <- y.big
     out$call <- cl
@@ -1047,6 +1140,7 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
     out$n.post <- n.post.samples
     out$n.thin <- n.thin
     out$n.burn <- n.burn
+    out$n.chains <- n.chains
 
     # K-fold cross-validation -------
     if (!missing(k.fold)) {
@@ -1228,7 +1322,8 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
         storage.mode(n.neighbors) <- "integer"
         storage.mode(n.burn) <- "integer"
         storage.mode(n.thin) <- "integer"
-
+	curr.chain <- 1
+	storage.mode(curr.chain) <- "integer"
 
         out.fit <- .Call("spIntPGOccNNGP", y.fit, X.fit, X.p.fit, coords.fit, p.occ, p.det, p.det.long, 
 		         J.fit, J.long.fit, K.fit, n.obs.fit, n.obs.long.fit, n.data, 
@@ -1240,7 +1335,8 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
 		         Sigma.beta, sigma.alpha, phi.a, phi.b, sigma.sq.a, sigma.sq.b, 
 		         nu.a, nu.b, tuning.c, cov.model.indx,
 		         n.batch, batch.length, accept.rate,  
-		         n.omp.threads.fit, verbose.fit, n.report, n.burn, n.thin, n.post.samples)
+		         n.omp.threads.fit, verbose.fit, n.report, n.burn, n.thin, n.post.samples, 
+			 curr.chain, n.chains)
 
         out.fit$beta.samples <- mcmc(t(out.fit$beta.samples))
         colnames(out.fit$beta.samples) <- x.names
@@ -1250,6 +1346,10 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
         } else {
           colnames(out.fit$theta.samples) <- c('sigma.sq', 'phi', 'nu')
         }
+	# Don't need to reorder these, because nothing is re-ordered and it doesn't matter
+	# since you are just returning the model deviance value.  
+	out.fit$z.samples <- mcmc(t(out.fit$z.samples))
+	out.fit$psi.samples <- mcmc(t(out.fit$psi.samples))
         out.fit$w.samples <- mcmc(t(out.fit$w.samples))
         out.fit$X <- X.fit
         out.fit$X.p <- X.p.fit
@@ -1262,6 +1362,7 @@ spIntPGOcc <- function(occ.formula, det.formula, data, inits, priors,
         out.fit$n.post <- n.post.samples
         out.fit$n.thin <- n.thin
         out.fit$n.burn <- n.burn
+	out.fit$n.chains <- 1
 	class(out.fit) <- "spPGOcc"
 
         out.pred <- predict.spPGOcc(out.fit, X.0, coords.0, verbose = FALSE)
