@@ -55,20 +55,28 @@ extern "C" {
     int *XpRE = INTEGER(XpRE_r); 
     double *lambdaPsi = REAL(lambdaPsi_r); 
     double *lambdaP = REAL(lambdaP_r); 
-    double *muBeta = REAL(muBeta_r); 
-    double *muAlpha = REAL(muAlpha_r); 
-    double *SigmaBetaInv = REAL(SigmaBeta_r); 
-    double *SigmaAlphaInv = REAL(SigmaAlpha_r); 
-    double *sigmaSqPsiA = REAL(sigmaSqPsiA_r); 
-    double *sigmaSqPsiB = REAL(sigmaSqPsiB_r); 
-    double *sigmaSqPA = REAL(sigmaSqPA_r); 
-    double *sigmaSqPB = REAL(sigmaSqPB_r); 
     int pOcc = INTEGER(pocc_r)[0];
     int pDet = INTEGER(pdet_r)[0];
     int pOccRE = INTEGER(pOccRE_r)[0]; 
     int pDetRE = INTEGER(pDetRE_r)[0]; 
     int nOccRE = INTEGER(nOccRE_r)[0]; 
     int nDetRE = INTEGER(nDetRE_r)[0]; 
+    int ppDet = pDet * pDet;
+    int ppOcc = pOcc * pOcc; 
+    int nnDetRE = nDetRE * nDetRE; 
+    int nnOccRE = nOccRE * nOccRE; 
+    double *muBeta = (double *) R_alloc(pOcc, sizeof(double));   
+    F77_NAME(dcopy)(&pOcc, REAL(muBeta_r), &inc, muBeta, &inc);
+    double *muAlpha = (double *) R_alloc(pDet, sizeof(double));   
+    F77_NAME(dcopy)(&pDet, REAL(muAlpha_r), &inc, muAlpha, &inc);
+    double *SigmaBetaInv = (double *) R_alloc(ppOcc, sizeof(double));   
+    F77_NAME(dcopy)(&ppOcc, REAL(SigmaBeta_r), &inc, SigmaBetaInv, &inc);
+    double *SigmaAlphaInv = (double *) R_alloc(ppDet, sizeof(double));   
+    F77_NAME(dcopy)(&ppDet, REAL(SigmaAlpha_r), &inc, SigmaAlphaInv, &inc);
+    double *sigmaSqPsiA = REAL(sigmaSqPsiA_r); 
+    double *sigmaSqPsiB = REAL(sigmaSqPsiB_r); 
+    double *sigmaSqPA = REAL(sigmaSqPA_r); 
+    double *sigmaSqPB = REAL(sigmaSqPB_r); 
     int *nOccRELong = INTEGER(nOccRELong_r); 
     int *nDetRELong = INTEGER(nDetRELong_r); 
     int J = INTEGER(J_r)[0];
@@ -85,7 +93,6 @@ extern "C" {
     int nBurn = INTEGER(nBurn_r)[0]; 
     int nPost = INTEGER(nPost_r)[0]; 
     int status = 0; 
-    double *z = REAL(zStarting_r); 
     int thinIndx = 0;
     int sPost = 0;  
 
@@ -139,6 +146,9 @@ extern "C" {
     // Latent detection random effects
     double *alphaStar = (double *) R_alloc(nDetRE, sizeof(double)); 
     F77_NAME(dcopy)(&nDetRE, REAL(alphaStarStarting_r), &inc, alphaStar, &inc); 
+    // Latent Occurrence
+    double *z = (double *) R_alloc(J, sizeof(double));   
+    F77_NAME(dcopy)(&J, REAL(zStarting_r), &inc, z, &inc);
     // Auxiliary variables
     double *omegaDet = (double *) R_alloc(nObs, sizeof(double));
     double *omegaOcc = (double *) R_alloc(J, sizeof(double));
@@ -168,10 +178,6 @@ extern "C" {
     /**********************************************************************
      * Other initial starting stuff
      * *******************************************************************/
-    int ppDet = pDet * pDet;
-    int ppOcc = pOcc * pOcc; 
-    int nnDetRE = nDetRE * nDetRE; 
-    int nnOccRE = nOccRE * nOccRE; 
     int JpOcc = J * pOcc; 
     int JnOccRE = J * nOccRE; 
     int nObspDet = nObs * pDet;
