@@ -20,7 +20,8 @@ extern "C" {
 		SEXP zLongIndx_r, SEXP dataIndx_r, SEXP alphaIndx_r, 
 		SEXP muBeta_r, SEXP muAlpha_r, SEXP SigmaBeta_r, SEXP sigmaAlpha_r, 
 		SEXP nSamples_r, SEXP nThreads_r, SEXP verbose_r, SEXP nReport_r, 
-		SEXP nBurn_r, SEXP nThin_r, SEXP nPost_r){
+		SEXP nBurn_r, SEXP nThin_r, SEXP nPost_r, SEXP currChain_r, 
+		SEXP nChain_r){
    
     /**********************************************************************
      * Initial constants
@@ -45,21 +46,25 @@ extern "C" {
     double *X = REAL(X_r);
     // Sorted by parameter, then data set, site, visit
     double *Xp = REAL(Xp_r);
-    // Priors for regression coefficients
-    double *muBeta = REAL(muBeta_r); 
-    double *muAlpha = REAL(muAlpha_r); 
-    double *SigmaBetaInv = REAL(SigmaBeta_r); 
-    double sigmaAlpha = REAL(sigmaAlpha_r)[0]; 
     int pOcc = INTEGER(pOcc_r)[0];
     int pDet = INTEGER(pDet_r)[0];
     int nData = INTEGER(nData_r)[0]; 
     int *pDetLong = INTEGER(pDetLong_r); 
+    int ppDet = pDet * pDet;
+    int ppOcc = pOcc * pOcc; 
+    // Priors for regression coefficients
+    double *muBeta = (double *) R_alloc(pOcc, sizeof(double));   
+    F77_NAME(dcopy)(&pOcc, REAL(muBeta_r), &inc, muBeta, &inc);
+    double *muAlpha = (double *) R_alloc(pDet, sizeof(double));   
+    F77_NAME(dcopy)(&pDet, REAL(muAlpha_r), &inc, muAlpha, &inc);
+    double *SigmaBetaInv = (double *) R_alloc(ppOcc, sizeof(double));   
+    F77_NAME(dcopy)(&ppOcc, REAL(SigmaBeta_r), &inc, SigmaBetaInv, &inc);
+    double sigmaAlpha = REAL(sigmaAlpha_r)[0]; 
     int J = INTEGER(J_r)[0];
     int *JLong = INTEGER(JLong_r); 
     int *K = INTEGER(K_r); 
     int *zLongIndx = INTEGER(zLongIndx_r); 
     int nObs = INTEGER(nObs_r)[0]; 
-    // Rprintf("nObs: %i\n", nObs); 
     int *nObsLong = INTEGER(nObsLong_r); 
     int *dataIndx = INTEGER(dataIndx_r); 
     int *alphaIndx = INTEGER(alphaIndx_r); 
@@ -68,11 +73,11 @@ extern "C" {
     int nBurn = INTEGER(nBurn_r)[0]; 
     int nPost = INTEGER(nPost_r)[0]; 
     int nThreads = INTEGER(nThreads_r)[0];
+    int currChain = INTEGER(currChain_r)[0];
+    int nChain = INTEGER(nChain_r)[0];
     int verbose = INTEGER(verbose_r)[0];
     int nReport = INTEGER(nReport_r)[0]; 
     int status = 0; 
-    // z starting values 
-    double *z = REAL(zStarting_r); 
     // For looping through data sets
     int stNObs = 0; 
     int stAlpha = 0; 
@@ -92,20 +97,26 @@ extern "C" {
      * Print Information 
      * *******************************************************************/
     if(verbose){
-      Rprintf("----------------------------------------\n");
-      Rprintf("\tModel description\n");
-      Rprintf("----------------------------------------\n");
-      Rprintf("Integrated Occupancy Model with Polya-Gamma latent\nvariable fit with %i sites.\n\n", J);
-      Rprintf("Integrating %i occupancy data sets.\n\n", nData); 
-      Rprintf("Number of MCMC samples: %i\n", nSamples);
-      Rprintf("Burn-in: %i \n", nBurn); 
-      Rprintf("Thinning Rate: %i \n", nThin); 
-      Rprintf("Total Posterior Samples: %i \n\n", nPost); 
+      if (currChain == 1) {
+        Rprintf("----------------------------------------\n");
+        Rprintf("\tModel description\n");
+        Rprintf("----------------------------------------\n");
+        Rprintf("Integrated Occupancy Model with Polya-Gamma latent\nvariable fit with %i sites.\n\n", J);
+        Rprintf("Integrating %i occupancy data sets.\n\n", nData); 
+        Rprintf("Samples per Chain: %i \n", nSamples);
+        Rprintf("Burn-in: %i \n", nBurn); 
+        Rprintf("Thinning Rate: %i \n", nThin); 
+	Rprintf("Number of Chains: %i \n", nChain);
+        Rprintf("Total Posterior Samples: %i \n\n", nPost * nChain); 
 #ifdef _OPENMP
-      Rprintf("\nSource compiled with OpenMP support and model fit using %i thread(s).\n\n", nThreads);
+        Rprintf("\nSource compiled with OpenMP support and model fit using %i thread(s).\n\n", nThreads);
 #else
-      Rprintf("Source not compiled with OpenMP support.\n\n");
+        Rprintf("Source not compiled with OpenMP support.\n\n");
 #endif
+      }
+      Rprintf("----------------------------------------\n");
+      Rprintf("\tChain %i\n", currChain);
+      Rprintf("----------------------------------------\n");
       Rprintf("Sampling ... \n");
     }
 
@@ -118,6 +129,9 @@ extern "C" {
     // Detection covariates
     double *alpha = (double *) R_alloc(pDet, sizeof(double));   
     F77_NAME(dcopy)(&pDet, REAL(alphaStarting_r), &inc, alpha, &inc);
+    // Latent Occurrence
+    double *z = (double *) R_alloc(J, sizeof(double));   
+    F77_NAME(dcopy)(&J, REAL(zStarting_r), &inc, z, &inc);
     // Auxiliary variables
     double *omegaDet = (double *) R_alloc(nObs, sizeof(double));
     double *omegaOcc = (double *) R_alloc(J, sizeof(double));
@@ -141,8 +155,6 @@ extern "C" {
     /**********************************************************************
      * Other initial starting stuff
      * *******************************************************************/
-    int ppDet = pDet * pDet;
-    int ppOcc = pOcc * pOcc; 
     int JpOcc = J * pOcc; 
     int nObspDet = nObs * pDet;
     double *tmp_ppDet = (double *) R_alloc(ppDet, sizeof(double));
