@@ -26,7 +26,7 @@ extern "C" {
 		 SEXP nuA_r, SEXP nuB_r, SEXP tuning_r, 
 		 SEXP covModel_r, SEXP nBatch_r, SEXP batchLength_r, SEXP acceptRate_r, 
 	         SEXP nThreads_r, SEXP verbose_r, SEXP nReport_r, SEXP nBurn_r, 
-		 SEXP nThin_r, SEXP nPost_r){
+		 SEXP nThin_r, SEXP nPost_r, SEXP currChain_r, SEXP nChain_r){
    
     /**********************************************************************
      * Initial constants
@@ -53,10 +53,16 @@ extern "C" {
     double *coordsD = REAL(coordsD_r); 
     // Xp is sorted by parameter then site, then visit (parameter 1 site 1, p1 site 2, etc) 
     double *Xp = REAL(Xp_r);
+    int pOcc = INTEGER(pocc_r)[0];
+    int pDet = INTEGER(pdet_r)[0];
+    int ppDet = pDet * pDet;
+    int ppOcc = pOcc * pOcc; 
     double *muBetaComm = REAL(muBetaComm_r); 
     double *muAlphaComm = REAL(muAlphaComm_r); 
-    double *SigmaBetaCommInv = REAL(SigmaBetaComm_r); 
-    double *SigmaAlphaCommInv = REAL(SigmaAlphaComm_r); 
+    double *SigmaBetaCommInv = (double *) R_alloc(ppOcc, sizeof(double));   
+    F77_NAME(dcopy)(&ppOcc, REAL(SigmaBetaComm_r), &inc, SigmaBetaCommInv, &inc);
+    double *SigmaAlphaCommInv = (double *) R_alloc(ppDet, sizeof(double));   
+    F77_NAME(dcopy)(&ppDet, REAL(SigmaAlphaComm_r), &inc, SigmaAlphaCommInv, &inc);
     double *tauSqBetaA = REAL(tauSqBetaA_r); 
     double *tauSqBetaB = REAL(tauSqBetaB_r); 
     double *tauSqAlphaA = REAL(tauSqAlphaA_r); 
@@ -70,8 +76,6 @@ extern "C" {
     double *tuning = REAL(tuning_r); 
     int covModel = INTEGER(covModel_r)[0];
     std::string corName = getCorName(covModel);
-    int pOcc = INTEGER(pocc_r)[0];
-    int pDet = INTEGER(pdet_r)[0];
     int J = INTEGER(J_r)[0];
     double *K = REAL(K_r); 
     int nObs = INTEGER(nObs_r)[0];
@@ -85,13 +89,13 @@ extern "C" {
     int nThin = INTEGER(nThin_r)[0];
     int nBurn = INTEGER(nBurn_r)[0]; 
     int nPost = INTEGER(nPost_r)[0]; 
+    int currChain = INTEGER(currChain_r)[0];
+    int nChain = INTEGER(nChain_r)[0];
     double acceptRate = REAL(acceptRate_r)[0];
     int nThreads = INTEGER(nThreads_r)[0];
     int verbose = INTEGER(verbose_r)[0];
     int nReport = INTEGER(nReport_r)[0];
     int status = 0; 
-    // Sorted by site, then species (e.g., site 1 sp 1, site 1 sp2, ...)
-    double *z = REAL(zStarting_r); 
     int thinIndx = 0; 
     int sPost = 0; 
 
@@ -108,22 +112,27 @@ extern "C" {
      * Print Information 
      * *******************************************************************/
     if(verbose){
-      // Rprintf allows you to print messages and value on the R console screen. 
-      Rprintf("----------------------------------------\n");
-      Rprintf("\tModel description\n");
-      Rprintf("----------------------------------------\n");
-      Rprintf("Spatial Multispecies Occupancy Model with Polya-Gamma latent\nvariable fit with %i sites and %i species.\n\n", J, N);
-      Rprintf("Number of MCMC samples: %i (%i batches of length %i)\n", nSamples, nBatch, batchLength);
-      Rprintf("Burn-in: %i \n", nBurn); 
-      Rprintf("Thinning Rate: %i \n", nThin); 
-      Rprintf("Total Posterior Samples: %i \n\n", nPost); 
-      Rprintf("Using the %s spatial correlation model.\n\n", corName.c_str());
+      if (currChain == 1) {
+        Rprintf("----------------------------------------\n");
+        Rprintf("\tModel description\n");
+        Rprintf("----------------------------------------\n");
+        Rprintf("Spatial Multispecies Occupancy Model with Polya-Gamma latent\nvariable fit with %i sites and %i species.\n\n", J, N);
+        Rprintf("Samples per chain: %i (%i batches of length %i)\n", nSamples, nBatch, batchLength);
+        Rprintf("Burn-in: %i \n", nBurn); 
+        Rprintf("Thinning Rate: %i \n", nThin); 
+        Rprintf("Number of Chains: %i \n", nChain);
+        Rprintf("Total Posterior Samples: %i \n\n", nPost * nChain); 
+        Rprintf("Using the %s spatial correlation model.\n\n", corName.c_str());
 #ifdef _OPENMP
-      Rprintf("\nSource compiled with OpenMP support and model fit using %i thread(s).\n\n", nThreads);
+        Rprintf("\nSource compiled with OpenMP support and model fit using %i thread(s).\n\n", nThreads);
 #else
-      Rprintf("Source not compiled with OpenMP support.\n\n");
+        Rprintf("Source not compiled with OpenMP support.\n\n");
 #endif
-      Rprintf("Adaptive Metropolis with target acceptance rate: %.1f\n", 100*acceptRate);
+        Rprintf("Adaptive Metropolis with target acceptance rate: %.1f\n", 100*acceptRate);
+      }
+      Rprintf("----------------------------------------\n");
+      Rprintf("\tChain %i\n", currChain);
+      Rprintf("----------------------------------------\n");
       Rprintf("Sampling ... \n");
     }
 
@@ -134,8 +143,6 @@ extern "C" {
     int pDetN = pDet * N; 
     int nObsN = nObs * N; 
     int JN = J * N;
-    int ppDet = pDet * pDet;
-    int ppOcc = pOcc * pOcc; 
     int JpOcc = J * pOcc; 
     int nObspDet = nObs * pDet;
     int JJ = J * J; 
@@ -188,6 +195,9 @@ extern "C" {
     // Spatial smoothing parameter for Matern
     double *nu = (double *) R_alloc(N, sizeof(double)); 
     F77_NAME(dcopy)(&N, REAL(nuStarting_r), &inc, nu, &inc); 
+    // Latent Occurrence
+    double *z = (double *) R_alloc(JN, sizeof(double));   
+    F77_NAME(dcopy)(&JN, REAL(zStarting_r), &inc, z, &inc);
     // Auxiliary variables
     // Only need to set aside J locations since you don't save these 
     // for each species
