@@ -74,7 +74,8 @@ extern "C" {
 		       SEXP sigmaSqPA_r, SEXP sigmaSqPB_r, 
 		       SEXP tuning_r, SEXP covModel_r, SEXP nBatch_r, SEXP batchLength_r, 
 		       SEXP acceptRate_r, SEXP nThreads_r, SEXP verbose_r, SEXP nReport_r, 
-		       SEXP nBurn_r, SEXP nThin_r, SEXP nPost_r){
+		       SEXP nBurn_r, SEXP nThin_r, SEXP nPost_r, SEXP currChain_r, 
+		       SEXP nChain_r){
    
     /**********************************************************************
      * Initial constants
@@ -105,10 +106,16 @@ extern "C" {
     int m = INTEGER(m_r)[0]; 
     // Xp is sorted by parameter then site, then visit (parameter 1 site 1, p1 site 2, etc) 
     double *Xp = REAL(Xp_r);
+    int pOcc = INTEGER(pocc_r)[0];
+    int pDet = INTEGER(pdet_r)[0];
+    int ppDet = pDet * pDet;
+    int ppOcc = pOcc * pOcc; 
     double *muBetaComm = REAL(muBetaComm_r); 
     double *muAlphaComm = REAL(muAlphaComm_r); 
-    double *SigmaBetaCommInv = REAL(SigmaBetaComm_r); 
-    double *SigmaAlphaCommInv = REAL(SigmaAlphaComm_r); 
+    double *SigmaBetaCommInv = (double *) R_alloc(ppOcc, sizeof(double));   
+    F77_NAME(dcopy)(&ppOcc, REAL(SigmaBetaComm_r), &inc, SigmaBetaCommInv, &inc);
+    double *SigmaAlphaCommInv = (double *) R_alloc(ppDet, sizeof(double));   
+    F77_NAME(dcopy)(&ppDet, REAL(SigmaAlphaComm_r), &inc, SigmaAlphaCommInv, &inc);
     double *tauSqBetaA = REAL(tauSqBetaA_r); 
     double *tauSqBetaB = REAL(tauSqBetaB_r); 
     double *tauSqAlphaA = REAL(tauSqAlphaA_r); 
@@ -129,8 +136,6 @@ extern "C" {
     int *uiIndx = INTEGER(uiIndx_r);
     int covModel = INTEGER(covModel_r)[0];
     std::string corName = getCorName(covModel);
-    int pOcc = INTEGER(pocc_r)[0];
-    int pDet = INTEGER(pdet_r)[0];
     int pDetRE = INTEGER(pDetRE_r)[0]; 
     int nDetRE = INTEGER(nDetRE_r)[0]; 
     int *nDetRELong = INTEGER(nDetRELong_r); 
@@ -145,14 +150,14 @@ extern "C" {
     int nSamples = nBatch * batchLength; 
     int nThin = INTEGER(nThin_r)[0];
     int nBurn = INTEGER(nBurn_r)[0]; 
-    int nPost = INTEGER(nPost_r)[0]; 
+    int nPost = INTEGER(nPost_r)[0];
+    int currChain = INTEGER(currChain_r)[0];
+    int nChain = INTEGER(nChain_r)[0];
     double acceptRate = REAL(acceptRate_r)[0];
     int nThreads = INTEGER(nThreads_r)[0];
     int verbose = INTEGER(verbose_r)[0];
     int nReport = INTEGER(nReport_r)[0];
     int status = 0; 
-    // Sorted by site, then species (e.g., site 1 sp 1, site 1 sp2, ...)
-    double *z = REAL(zStarting_r); 
     int thinIndx = 0; 
     int sPost = 0; 
 
@@ -169,23 +174,28 @@ extern "C" {
      * Print Information 
      * *******************************************************************/
     if(verbose){
-      Rprintf("----------------------------------------\n");
-      Rprintf("\tModel description\n");
-      Rprintf("----------------------------------------\n");
-      Rprintf("NNGP Multispecies Occupancy Model with Polya-Gamma latent\nvariable fit with %i sites and %i species.\n\n", J, N);
-      Rprintf("Number of MCMC samples: %i (%i batches of length %i)\n", nSamples, nBatch, batchLength);
-      Rprintf("Burn-in: %i \n", nBurn); 
-      Rprintf("Thinning Rate: %i \n", nThin); 
-      Rprintf("Total Posterior Samples: %i \n\n", nPost); 
-      Rprintf("Using the %s spatial correlation model.\n\n", corName.c_str());
-      Rprintf("Using %i nearest neighbors.\n\n", m);
-
+      if (currChain == 1) {
+        Rprintf("----------------------------------------\n");
+        Rprintf("\tModel description\n");
+        Rprintf("----------------------------------------\n");
+        Rprintf("NNGP Multispecies Occupancy Model with Polya-Gamma latent\nvariable fit with %i sites and %i species.\n\n", J, N);
+        Rprintf("Samples per chain: %i (%i batches of length %i)\n", nSamples, nBatch, batchLength);
+        Rprintf("Burn-in: %i \n", nBurn); 
+        Rprintf("Thinning Rate: %i \n", nThin); 
+        Rprintf("Number of Chains: %i \n", nChain);
+        Rprintf("Total Posterior Samples: %i \n\n", nPost * nChain); 
+        Rprintf("Using the %s spatial correlation model.\n\n", corName.c_str());
+        Rprintf("Using %i nearest neighbors.\n\n", m);
 #ifdef _OPENMP
-      Rprintf("Source compiled with OpenMP support and model fit using %i thread(s).\n\n", nThreads);
+        Rprintf("Source compiled with OpenMP support and model fit using %i thread(s).\n\n", nThreads);
 #else
-      Rprintf("Source not compiled with OpenMP support.\n\n");
+        Rprintf("Source not compiled with OpenMP support.\n\n");
 #endif
-      Rprintf("Adaptive Metropolis with target acceptance rate: %.1f\n", 100*acceptRate);
+        Rprintf("Adaptive Metropolis with target acceptance rate: %.1f\n", 100*acceptRate);
+      }
+      Rprintf("----------------------------------------\n");
+      Rprintf("\tChain %i\n", currChain);
+      Rprintf("----------------------------------------\n");
       Rprintf("Sampling ... \n");
     }
 
@@ -197,8 +207,6 @@ extern "C" {
     int nObsN = nObs * N; 
     int nDetREN = nDetRE * N; 
     int JN = J * N;
-    int ppDet = pDet * pDet;
-    int ppOcc = pOcc * pOcc; 
     int JpOcc = J * pOcc; 
     int nObspDet = nObs * pDet;
     int JJ = J * J;
@@ -259,6 +267,9 @@ extern "C" {
     // Spatial smoothing parameter for Matern
     double *nu = (double *) R_alloc(N, sizeof(double)); 
     F77_NAME(dcopy)(&N, REAL(nuStarting_r), &inc, nu, &inc); 
+    // Latent Occurrence
+    double *z = (double *) R_alloc(JN, sizeof(double));   
+    F77_NAME(dcopy)(&JN, REAL(zStarting_r), &inc, z, &inc);
     // Auxiliary variables
     // Only need to set aside J locations since you don't save these 
     // for each species
